import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az
import pandas as pd
import pickle

pd.options.display.max_rows = 1000
pd.options.display.max_columns = 1000
 
def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]      
    return MAP_Est, est_pdf[map_idx]


with open('i_data.pkl', 'rb') as f:
    inference_data = pickle.load(f)
with open('d_frame.pkl', 'rb') as f:
    fit = pickle.load(f)
with open('K.pkl', 'rb') as f:
   (K, n_n, n_s, s_n, s_s) = pickle.load(f)

smry = az.summary(inference_data, var_names = ['mu_s', 'sgm_s'])
print('Summary =\n',smry)

mu_s_map = CalcMAPEst(fit['mu_s'])[0]
sb.kdeplot(fit['mu_s'])
plt.xlabel('$\mu_s$')
plt.title('$\mu_s$(MAP) = {0:.3f}'.format(mu_s_map))
plt.savefig('Fig_mu.png')
plt.show()

sgm_s_map = CalcMAPEst(fit['sgm_s'])[0]
sb.kdeplot(fit['sgm_s'])
plt.title('$\sigma_s$(MAP) = {0:.3f}'.format(sgm_s_map))
plt.xlabel('$\sigma_s$')
plt.savefig('Fig_sgm.png')
plt.show()

print('\n I am calculating.')

s_title = ''
if K % 2 == 1:
    for k in range(K-1):
        Csmpl = fit[f'C.{k+1}']
        sb.kdeplot(Csmpl)
        c_map = CalcMAPEst(Csmpl)[0]
        s_title += f'C{k+1}={c_map:.2f}'
        if k < K-2:
            s_title += ', '
    
else:
    for k in range(K-1):
        if k != (K//2) - 1:
            Csmpl = fit[f'C.{k+1}']
            sb.kdeplot(Csmpl)
            c_map = CalcMAPEst(Csmpl)[0]
            s_title += f'C{k+1}={c_map:.2f}'
            if k < K-2:
                s_title += ', '
        else:
            plt.plot([0], [0], marker = 'o', markersize = 15, c = 'r')
            s_title += f'C{k+1}=0, '
            
plt.title('MAP estimates\n' + s_title)
plt.savefig('FigC.png')
plt.show()

b_map = CalcMAPEst(fit['b'])[0]
sb.kdeplot(fit['b'])
plt.title('b(MAP) = {0:.3f}'.format(b_map))
plt.savefig('Fig_b.png')
plt.show()

cum_n_n = np.cumsum(n_n)
pcum_n_n = cum_n_n/cum_n_n[-1]
cum_n_s = np.cumsum(n_s)
pcum_n_s = cum_n_s / cum_n_s[-1]
cum_s_n = np.cumsum(s_n)
pcum_s_n = cum_s_n / cum_s_n[-1]
cum_s_s = np.cumsum(s_s)
pcum_s_s = cum_s_s / cum_s_s[-1]

est_pcum_n_n = np.empty(K-1)
for k in range(K-1):
    est_pcum_n_n[k] = np.median(fit[f'nn_cum_p.{k+2}'])
est_pcum_n_s = np.empty(K-1)
for k in range(K-1):
    est_pcum_n_s[k] = np.median(fit[f'ns_cum_p.{k+2}'])
est_pcum_s_n = np.empty(K-1)
for k in range(K-1):
    est_pcum_s_n[k] = np.median(fit[f'sn_cum_p.{k+2}'])
est_pcum_s_s = np.empty(K-1)
for k in range(K-1):
    est_pcum_s_s[k] = np.median(fit[f'ss_cum_p.{k+2}'])

plt.plot(est_pcum_n_n, pcum_n_n[:-1], label = 'n_n')
plt.plot(est_pcum_n_s, pcum_n_s[:-1], label = 'n_s')
plt.plot(est_pcum_s_n, pcum_s_n[:-1], label = 's_n')
plt.plot(est_pcum_s_s, pcum_s_s[:-1], label = 's_s')
plt.plot([0,1], [0,1], c = 'k', ls = '--', label = 'Obs.=Est.')
plt.xlabel('Est.Cum.P')
plt.ylabel('Obs.Cum.P')
plt.legend()
plt.title('Cumulative Proportions')
plt.savefig('FigCumEstObs.png')
plt.show()


print('End.')
